////////////////////////////////////////////////////////////////////////
//
//  CAL PreHit class in ROOT
//
//  2004.10.16   Allister Sanchez  Original version.
//
// $Id: S4CALPreHit.cxx,v 1.9 2006/07/03 07:14:16 miyamoto Exp $
////////////////////////////////////////////////////////////////////////

#include "S4CALPreHit.h"

#include <iostream>
using namespace std;

ClassImp(S4CALPreHit);

//---------------------------------------------------------------------
S4CALPreHit::S4CALPreHit()
            : S4VExactHit(0,                 // hit ID
                          0,                 // track ID
                          0,                 // mother track ID
                          0,                 // PDG code
                          0,                 // charge
                          TVector3(0.,0.,0.),// position
                          0,                 // TOF
                          TVector3(0.,0.,0.),// momentum
                          0,                 // energy
                          0,                 // energy deposit
                          0),                // address ptr 
              fBreakPointPtr(0)
{
}


//---------------------------------------------------------------------
S4CALPreHit::S4CALPreHit(Int_t     preHitID,      // prehit ID
			 Int_t     trackID,       // track ID 
			 Int_t     motherTrackID, // mother track ID
			 Int_t     pcode,         // PDG code
			 Double_t  charge,        // charge
			 TVector3  pos,           // Injection position [cm]
			 Double_t  tof,           // TOF      [nsec]
			 TVector3  mom,           // momentum [GeV] 
			 Double_t  etot,          // energy
			 Double_t  edep,          // energy deposit [GeV]      
                         TVAddress *p)            // address ptr)
            : S4VExactHit(preHitID,              // hit ID
			  trackID,               // track ID 
			  motherTrackID,         // mother track ID
			  pcode,                 // PDG code
			  charge,                // charge
			  pos,                   // Injection position [cm]
			  tof,                   // TOF      [nsec]
			  mom,                   // momentum [GeV] 
			  etot,                  // energy
			  edep,                  // energy deposit [GeV] 
			  p),                    // address ptr)
              fBreakPointPtr(S4BreakPoint::GetBreakPointPtr(trackID))
{
  if (fBreakPointPtr) fBreakPointPtr->SetPHitPtr(this);
}

//--------------------------------------------------------------------
S4CALPreHit::~S4CALPreHit()
{
}

//--------------------------------------------------------------------
void S4CALPreHit::DebugPrint(Option_t *) const
{
  std::cout << "S4CALPreHit ================" << std::endl;
  std::cout << " PreHitID=" << GetPreHitID()
	    << " TrkID=" << GetTrackID()
	    << " TOF(ns)=" << GetTOF()
	    << " E(GeV)=" << GetTotalEnergy()
	    << " PDGcode=" << GetPDGCode()
	    << " Charge=" << GetCharge() << std::endl;
  std::cout << " Momentum(GeV)=(" << GetMomentum().x() << ", "
	    << GetMomentum().y() << ", "
	    << GetMomentum().z() << ") " << std::endl;
  std::cout << " InjectionPoint(cm)=(" << GetInjectionPoint().x() << ", "
	    << GetInjectionPoint().y() << ", "
	    << GetInjectionPoint().z() << ") " << std::endl;
}

//---------------------------------------------------------------
Int_t S4CALPreHit::Compare(const TObject *hit) const
{
  return 0;
}

//---------------------------------------------------------------
Bool_t S4CALPreHit::IsSortable() const
{
  return kFALSE;
}
