#ifndef S4BREAKPOINT_H 
#define S4BREAKPOINT_H 
//*************************************************************************
//* --------------------
//* class S4BreakPoint
//* --------------------
//* (Description)
//*     Class to store break point information equivalent to J4BreakPoint.
//*    
//* (Update Record)
//*     2006/01/29  S.Yamamoto   Original version.
//*************************************************************************

#include "TLorentzVector.h"
#include "TObjArray.h"
#include "TAttLockable.h"
#include <map>

// ************************************************************************
class S4BreakPoint : public TObjArray, public TAttLockable {
friend class S4HISTMCTruthBuf;
public:
  S4BreakPoint();
  S4BreakPoint(      Int_t     hitID,       // hit ID
                     Int_t     trackID,     // track ID
                     Int_t     mtrackID,    // mother track ID
                     Int_t     pcode,       // PDG code
		     Double_t  charge,      // charge
               const TVector3 &pos,         // position [cm]
                     Double_t  tof,         // TOF      [nsec]
               const TVector3 &mom,         // momentum [GeV]
	             Double_t  energy);     // energy [GeV]
  virtual ~S4BreakPoint();

  // Data Getters
  inline Int_t           GetTrackID()        const { return fTrackID;       }
  inline Int_t           GetParentID()       const { return fParentID;      }
  inline Double_t        GetCharge()         const { return fCharge;        }
  inline Int_t           GetParticleID()     const { return fParticleID;    }
  inline Double_t        GetGlobalTime()     const { return fGlobalTime;    }
  inline TVector3        GetPosition()       const { return fPosition;      }
  inline TLorentzVector  GetFourMomentum()   const { return fFourMomentum;  }
  inline Double_t        GetKineticEnergy()  const { return fKineticEnergy; }
  inline TObject        *GetPFOPtr()         const { return fPFOPtr;        }
  inline TObject        *GetPHitPtr()        const { return fPHitPtr;       }

  // Data Setter
   
  inline  void           SetPFOPtr (TObject *pfop) { fPFOPtr  = pfop; }
  inline  void           SetPHitPtr(TObject *phtp) { fPHitPtr = phtp; }
  virtual void           LockAllDescendants();
          void           DebugPrint(const Char_t *opt = "");

  static S4BreakPoint  * GetBreakPointPtr(Int_t trackID);
  static void            Clear();
  static void            PrintAll(const Char_t *opt = "");
  static std::map<Int_t, S4BreakPoint*>
                       & GetBreakPointMap();

private:
  Int_t           fTrackID;         // track ID
  Int_t           fParentID;        // parent track ID
  Double_t        fCharge;          // charge
  Int_t           fParticleID;      // PDG code
  Double_t        fGlobalTime;      // Time since the collision
  TVector3        fPosition;        // positon of break point
  TLorentzVector  fFourMomentum;    // 4-momentum
  Double_t        fKineticEnergy;   // kinetic energy
  TObject        *fPFOPtr;          // pointer to pfo
  TObject        *fPHitPtr;         // pointer to phit

  static std::map<Int_t, S4BreakPoint*> fgBreakPointMap; // TrackID to BreakPoint map

  ClassDef(S4BreakPoint, 3) // S4BreakPoint class
};

#endif
