//*****************************************************************************
//* ================
//* S4CALHitMakerBuf
//* ================
//*   METIS module to make (smeared) CAL hits.
//*   Mother class: S4VHit
//*
//* (Update Record)
//*   2004.07.31  A.Sanchez  Primitive version.
//*****************************************************************************
#include "S4CALHitMaker.h"
#include "S4CALHitMakerBuf.h"
#include "S4CALMCTruthBuf.h"
#include "S4CALExactHit.h"
#include "S4CALHit.h"
#include "S4CALPreHit.h"
#include "JSFSteer.h"

#include "TRandom.h"
#include <map>
#include <utility>
#include <iostream>
#include <vector>

ClassImp(S4CALHitMakerBuf);

Int_t    S4CALHitMakerBuf::fgScheme = 0;

S4CALHitMakerBuf::S4CALHitMakerBuf(S4CALHitMaker *hitmaker, 
                                   const Char_t  *name,
                                   const Char_t  *title,
				         Int_t    bufsize)
                : JSFEventBuf(name, title, hitmaker),
                  fOutputBufPtr(0),
                  fBufSize(bufsize)
{
}


S4CALHitMakerBuf::~S4CALHitMakerBuf()
{
   if (fOutputBufPtr) fOutputBufPtr->Delete();
   delete fOutputBufPtr;
}

//============================================================================
// MakeHits ------------------------------------------------------------------
Bool_t S4CALHitMakerBuf::MakeHits(TObjArray *hits)
{
  if (!hits) return kTRUE;
  if (!fOutputBufPtr) fOutputBufPtr = new TObjArray(fBufSize);
  if (fOutputBufPtr->GetEntries() > 0) fOutputBufPtr->Delete();
  fOutputBufPtr->SetOwner();

  if (!fgScheme) { // realistic (1 smeared hit consists of 1 or more exact hits)
    // loop over exact hits and merge those with the same cellID
    MergeHits(hits);
  } else { // idealistic (1 smeared hit corresponds to 1 exact hit)
    TIter next(hits);
    S4CALExactHit* ehp;
    while ((ehp = static_cast<S4CALExactHit*>(next()))) {
      if (ehp->GetEnergyDeposit() > 0.) {
        fOutputBufPtr->Add(new S4CALHit(ehp));
      }
    }
  }
  return kTRUE;
}

//============================================================================
// MergeHits -----------------------------------------------------------------
void S4CALHitMakerBuf::MergeHits(TObjArray* hits)
{
  S4CALExactHit::SetSortBy(S4CALExactHit::kCellID); // sort by Cell ID
  hits->UnSort();
  hits->Sort();

  Int_t     prevcellid = -999999;
  TIter next(hits);
  S4CALExactHit *ehp;
  S4CALHit      *curhp = 0;
  while ((ehp = static_cast<S4CALExactHit *>(next()))) {
    if (ehp->GetCellID() == prevcellid) {
      curhp->Merge(ehp);
    } else {
      curhp = new S4CALHit(ehp);
      fOutputBufPtr->Add(curhp);
    }
  }
}
