#include "S4PFObject.h"
#include "S4CALCluster.h"
#include "S4CALHit.h"
#include "S4CALExactHit.h"
#include "S4CALPreHit.h"
#include "S4BreakPoint.h"
#include "S4HYBTrack.h"

#if 0
#else
#include "TVirtualPad.h"
#include "TPolyLine3D.h"
#endif
#include "TString.h"
#include <iostream>

ClassImp(S4PFObject)

S4PFObject::S4PFObject(S4CALCluster *cp, S4HYBTrack *tp)
	  : fClusterPtr(cp),
	    fTrackPtr(tp)
{
  S4BreakPoint *bpp = 0;
  if (tp) {
    SetE(tp->GetEnergy());
    SetVect(tp->GetMomentum());
    bpp = tp->GetBreakPointPtr();
  } else if (cp) {
    SetE(cp->GetEnergy());
    SetVect(cp->GetMomentum());
    S4CALHit      *chp = static_cast<S4CALHit *>(cp->At(0));
    S4CALExactHit *ehp = static_cast<S4CALExactHit *>(chp->At(0));
    Int_t j4tkid = ehp->GetPreHitPtr()->GetTrackID();
                   bpp = S4BreakPoint::GetBreakPointPtr(j4tkid);
  }
  if (bpp) bpp->SetPFOPtr(this); 
}

S4PFObject::~S4PFObject()
{
}

void S4PFObject::Draw(Int_t color, const Char_t *opt)
{
  if (fClusterPtr) fClusterPtr->Draw(color, opt);
#if 0
  if (fTrackPtr  ) fTrackPtr  ->Draw(color, opt);
#else
  if (fTrackPtr && fClusterPtr) {
    if (!gPad) return;
    gPad->cd();

    static const Int_t kNmax = 3;
    TPolyLine3D *pm3dp = new TPolyLine3D(kNmax);
    pm3dp->SetBit(kCanDelete);
    pm3dp->SetLineColor(color);

    S4BreakPoint  *bpp = fTrackPtr->GetBreakPointPtr();
    S4CALHit      *chp = static_cast<S4CALHit *>(fClusterPtr->At(0));
    S4CALExactHit *ehp = static_cast<S4CALExactHit *>(chp->At(0));
    S4CALPreHit   *php = ehp->GetPreHitPtr();
    TVector3 xv[kNmax];
    if (kNmax == 2) {
      xv[0] = bpp->GetPosition();
      // xv[2] = php->GetPosition();
      Double_t r = php->GetPosition().Mag();
      xv[1] = xv[0] + r*bpp->GetFourMomentum().Vect().Unit();
    } else if (kNmax == 3) {
      xv[0] = bpp->GetPosition();
      xv[2] = php->GetPosition();
      xv[1] = xv[0] + ((xv[2]-xv[0]).Mag()/2.)*bpp->GetFourMomentum().Vect().Unit();
    }
#if 0
    std::cerr << "-----" << std::endl;
#endif
    for (Int_t i=0; i<kNmax; i++) {
      pm3dp->SetPoint(i, xv[i].X(), xv[i].Y(), xv[i].Z());
#if 0
      std::cerr << "(" << xv[i].X() << ","
                       << xv[i].Y() << ","
                       << xv[i].Z() << ") ";
#endif
    }
#if 0
    std::cerr << std::endl;
#endif
    pm3dp->Draw("l");
    gPad->Update();
  }
#endif
}

void S4PFObject::DebugPrint(const Option_t *opt)
{
  using namespace std;
  cerr << " -- S4PFObject::DebugPrint ---------------------- " << endl
       << "  clPtr = " << fClusterPtr                          << endl
       << "  tkPtr = " << fTrackPtr                            << endl
       << " ------------------------------------------------ " << endl;
  TString opts(opt);
  opts.ToLower();
  if (opts.Contains("detail")) {
    if (fTrackPtr)   fTrackPtr  ->DebugPrint(opt);
    if (fClusterPtr) fClusterPtr->DebugPrint(opt);
  }
}
