//*************************************************************************
//* --------------------
//* S4HYBTrackMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes pad Tracks.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2001/08/22  K.Hoshina	Original version.
//*************************************************************************

#include <iostream>
#include "S4HYBTrackMakerBuf.h"
#include "S4HYBTrack.h"
#include "S4BreakPoint.h"

//#define __DEBUG__

std::map<Int_t, S4HYBTrack *> S4HYBTrackMakerBuf::fJ4TrackMap;

ClassImp(S4HYBTrackMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
S4HYBTrackMakerBuf::S4HYBTrackMakerBuf(const Char_t    *name,
				       const Char_t    *title, 
				       S4HYBTrackMaker *module)
                   : JSFEventBuf(name, title, module),
                     fOutputBufPtr(0)
{
}	       
	       
S4HYBTrackMakerBuf::S4HYBTrackMakerBuf(S4HYBTrackMaker *module,
			               const Char_t    *name,
	                               const Char_t    *title) 
                   : JSFEventBuf(module, name, title),
                     fOutputBufPtr(0)
{
}	       

//=====================================================================
//* destructor --------------------------------------------------------
S4HYBTrackMakerBuf::~S4HYBTrackMakerBuf()
{
}

//=====================================================================
//* GetOutputBufPtr ---------------------------------------------------
TObjArray *S4HYBTrackMakerBuf::GetOutputBufPtr()
{
  if (!fOutputBufPtr) fOutputBufPtr = new TObjArray();
  return fOutputBufPtr;
}

//=====================================================================
//* MakeTracks ----------------------------------------------------------
void S4HYBTrackMakerBuf::MakeTracks()
{
  GetOutputBufPtr()->Delete();
  // Make tracks here.
  std::map<Int_t, S4BreakPoint *> &bpmap = S4BreakPoint::GetBreakPointMap();
  std::map<Int_t, S4BreakPoint *>::iterator iter;
  for (iter  = bpmap.begin(); iter != bpmap.end(); iter++) {
    if (iter->second->GetCharge()) {
       GetOutputBufPtr()->Add(new S4HYBTrack(iter->second));
    }
  }
  MakeMap();
}

//=====================================================================
//* MakeMap -----------------------------------------------------------
void S4HYBTrackMakerBuf::MakeMap()
{
  fJ4TrackMap.clear();
  TObjArray *htrksp = GetOutputBufPtr();
  if (!htrksp->GetEntries()) return;

  TIter next(htrksp);
  S4HYBTrack *htrkp = 0;
  while ((htrkp = static_cast<S4HYBTrack *>(next()))) {
    Int_t j4trkid = htrkp->GetTrackID();
    fJ4TrackMap[j4trkid] = htrkp;
#if 0
    std::cerr << "j4trkid = " << j4trkid << " htrkp = " << htrkp << std::endl;
#endif
  }
}

//=====================================================================
//* GetHYBTrackPtr ----------------------------------------------------
S4HYBTrack *S4HYBTrackMakerBuf::GetHYBTrackPtr(Int_t j4trkid)
{
  std::map<Int_t,S4HYBTrack *>::iterator iter = fJ4TrackMap.find(j4trkid);
  if (iter != fJ4TrackMap.end()) {
    return static_cast<S4HYBTrack *>(iter->second);
  } else {
    return 0;
  }
}
