// $Id: J4CALSD.cc,v 1.17 2007/10/02 08:58:20 fujiik Exp $
//*************************************************************************
//* --------------------
//* J4CALSD
//* --------------------
//* (Description)
//* 	Class for describing his/her sensitive detector.
//*     
//* (Update Record)
//*	2000/12/08  K.Hoshina	Original version.
//*************************************************************************
#include "G4Version.hh"
#include "J4CALSD.hh"
#include "J4CALPreHit.hh"
#include "J4CALPreHitKeeper.hh"
#include "J4CAL.hh"
#include "J4TrackingAction.hh"
#include "J4HistoryKeeper.hh"
#include <cmath>
 
//=====================================================================
//---------------------
// class definition
//---------------------

//=====================================================================
//* constructor -------------------------------------------------------
J4CALSD::J4CALSD( J4VDetectorComponent* detector )
        :J4VSD<J4CALPreHit>( detector )
{
  J4TrackingAction::GetInstance()->Add(J4CALPreHitKeeper::GetInstance());
  J4TrackingAction::GetInstance()->Add(J4HistoryKeeper::GetInstance());
  J4HistoryKeeper::GetInstance()->SetPHitKeeperPtr(J4CALPreHitKeeper::GetInstance());
}

//=====================================================================
//* destructor --------------------------------------------------------
J4CALSD::~J4CALSD()
{
}

//=====================================================================
//* Initialize --------------------------------------------------------
void J4CALSD::Initialize( G4HCofThisEvent* PreHCTE )
{
  //create hit collection(s) and
  //push H.C. to "Hit Collection of This Event"
  
  MakeHitBuf( PreHCTE );
}

//=====================================================================
//* ProcessHits -------------------------------------------------------
G4bool J4CALSD::ProcessHits( G4Step* aStep, G4TouchableHistory* /* ROhist */ )
{
  //In order to use Get function, you must call SetNewStep() at first.
  SetNewStep(aStep);

  //Get perticle information
  const G4ThreeVector  &pre           = GetPrePosition();
  const G4ThreeVector  &momentum      = GetMomentum();
  G4double              energy        = GetKineticEnergy();
  G4double              tof           = GetTof();
  G4int                 trackID       = GetTrackID();
  G4int                 motherTrackID = GetMotherTrackID();
  G4ParticleDefinition *particle      = GetParticle(); 
  
  // Create new pre hit --------------------------------------
  J4CALPreHitKeeper *phkp = J4CALPreHitKeeper::GetInstance();
  if (phkp->IsNext()) { 
     phkp->IncrementCurrentPreHitID();
     G4int preHitID = phkp->GetCurrentPreHitID();
     // create new hit!
     J4CALPreHit *preHitPtr = new J4CALPreHit(GetComponent(), 
		                              preHitID, 
					      pre, 
					      momentum, 
					      energy, 
					      tof, 
					      trackID,
					      particle, 
					      motherTrackID);
     phkp->SetCurrentPreHitPtr(preHitPtr);
     static_cast<J4CALPreHitBuf *>(GetHitBuf())->insert(preHitPtr);
  }
  return true;
}

//=====================================================================
//* DrawAll -----------------------------------------------------------
void J4CALSD::DrawAll()
{
}

//=====================================================================
//* PrintAll ----------------------------------------------------------
void J4CALSD::PrintAll()
{
  G4int nHit= ( (J4CALPreHitBuf*)GetHitBuf() ) -> entries();
  std::cout << "------------------------------------------" << std::endl
            << "*** tracker Hit (#hits=" << nHit << ")" << std::endl;
  ( (J4CALPreHitBuf*)GetHitBuf() ) -> PrintAllHits();
}
