// $Id: J4EventAction.cc,v 1.10 2007/05/13 06:31:03 miyamoto Exp $
// ====================================================================
//      J4EventAction.cc
//
//                                 .JLC-CDC, 2001
// ====================================================================
#include <sstream>
#include <fstream>
#include <iomanip>
#include "J4Timer.hh"

#include "G4RunManager.hh"
#include "G4Event.hh"
#include "G4SDManager.hh"

#include "J4RunAction.hh"
#include "J4EventAction.hh"
#include "J4TrackingAction.hh"

#include "G4VHitsCollection.hh"

#include "J4DetectorConstruction.hh"
#include "J4HistoryKeeper.hh"
#include "J4BreakPoint.hh"

// SD of each detector componet

// Visualization
#include "G4UImanager.hh"
#include "G4VVisManager.hh"
  
// ====================================================================
//
// class description
//
// ====================================================================

G4int J4EventAction::fEventID=0;

////////////////////////////////////
J4EventAction::J4EventAction()
////////////////////////////////////
{
  //   std::cerr << "J4EventAction::constructor called" << std::endl;
   G4int timerid = -1;
   G4String classname("J4EventAction");
   G4String timername("EventTimer");
   fEventTimer = new J4Timer(timerid, classname, timername);
}

////////////////////////////////////
J4EventAction::~J4EventAction()
////////////////////////////////////
{
   delete fEventTimer;
}

/////////////////////////////////////////////////////////////////
void J4EventAction::BeginOfEventAction(const G4Event* anEvent)
/////////////////////////////////////////////////////////////////
{
  // printout primary information
  std::cerr << "**********************************************" << std::endl;
  std::cerr << "**** Begin of _event_number_" 
            << anEvent->GetEventID() << "_*******" <<std::endl;
  std::cerr << "**********************************************" << std::endl;

  J4DetectorConstruction::GetEXPHall()->UnlockOutput();
  fEventTimer->Start();
  // =====================================================
  J4TrackingAction::GetInstance()->ResetTrackCounter();
  J4TrackingAction::GetInstance()->Clear();   
  // =====================================================

   fEventID = anEvent->GetEventID();
}

///////////////////////////////////////////////////////////////
void J4EventAction::EndOfEventAction(const G4Event* anEvent)
///////////////////////////////////////////////////////////////
{
  const G4RunManager* runManager= G4RunManager::GetRunManager();
  const G4UserRunAction* usrRunAction= runManager-> GetUserRunAction();

  // ====================================================================  
  // printout hit information of each detector

     G4int event = anEvent->GetEventID();
     std::cerr << "Start EndOfEventAction of event " << event << "   " << std::endl;
     
   std::ofstream& ofs= ((J4RunAction*)usrRunAction)-> GetOutputFileStream();
   if (!ofs.good()) {
     G4String errorMessage= "EventAction::EndOfEventAction(): write error.";
     G4Exception(errorMessage);
   } else {
     ofs << "*******_Start_of_event_" << event << "_*******" <<std::endl;
     ofs << event << std::endl;

     // ====================================================================  
     // output "J4BreakPointMap"
#ifdef __DEBUG__
     J4BreakPoint::DebugPrint("debug"); 
#endif
     J4BreakPoint::Output(); 

     // ====================================================================  
     // output "Hit Collection of This Event" to a hit data file
  
     G4HCofThisEvent* HCTE= anEvent-> GetHCofThisEvent();
     if (!HCTE) return;  // no hits in this events. nothing to do!
  
     J4DetectorConstruction::GetEXPHall()->OutputAll(HCTE, ofs);

     ofs << "*******_End_of_event_" << event << "_*******" <<std::endl;
   }

   fEventTimer->Stop();
   std::cerr << "*******_event_"<< event << "_finished *******" <<std::endl;

   // ---------------------------------------------------------------------
   // end of event.......

   using namespace CLHEP;

   std::stringstream tmpstr;
   tmpstr << "seeds/seed.evt." << std::setw(5) << std::setfill('0')
          << event << std::ends;
   HepRandom::saveEngineStatus(tmpstr.str().data());
}
