//$Id: 
//*************************************************************************
//* --------------------
//* J4TrackingAction
//* --------------------
//* (Description)
//*    Derivated class for UserTrackingAction. 
//*
//* (Update Record)
//*     2000/12/08  K.Hoshina   Original version.
//*************************************************************************

#include "G4EventManager.hh"
#include "G4Event.hh"
#include "G4HCtable.hh"
#include "G4SDManager.hh"

#include "J4TrackingAction.hh"
#if 0
#include "J4TrackingActionMessenger.hh"
#endif
#include "G4TrackingManager.hh"
#include "G4Track.hh"
#include "J4BreakPoint.hh"
#include <vector>
#include "J4Timer.hh"

J4TrackingAction                   *J4TrackingAction::fgInstance = 0;
//=====================================================================
//* Constructor -------------------------------------------------------
J4TrackingAction::J4TrackingAction()
                : fCurrentTrackID(std::numeric_limits<int>::max()),
                  fStoredTrajectoryID(1),
#if 0
                  fMessenger(0),
#endif
                  fTrackCounts(std::numeric_limits<int>::min()) 
{
   if (fgInstance) {
      G4cerr << ">>>>>>> Error in J4TrackingAction::J4TrackingAction" << G4endl
             << "  duplicateing a singleton!"                         << G4endl
             << "  abort"                                             << G4endl;
      ::abort();
   }
#if 0
   fMessenger = new J4TrackingActionMessenger(this);
#endif

   G4int timerid = -1;
   G4String classname("J4TrackingAction");
   G4String timername("J4TrackingTimer");
   fTrackingTimer = new J4Timer(timerid, classname, timername);

   G4int timerid1 = -1;
   G4String cname1("J4TrackingActionTracker");
   G4String tname1("TrackerTimer");
   fTimerTracker=new J4Timer(timerid1, cname1, tname1);

   G4int timerid2 = -1;
   G4String cname2("J4TrackingActionCal");
   G4String tname2("CalTimer");
   fTimerCal=new J4Timer(timerid2, cname2, tname2);

   fgInstance = this;
}

//=====================================================================
//* Destructor -------------------------------------------------------
J4TrackingAction::~J4TrackingAction()
{
#if 0
   if (fMessenger) delete fMessenger;
#endif
}

//=====================================================================
//* PreUserTrackingAction ---------------------------------------------
void J4TrackingAction::PreUserTrackingAction(const G4Track* aTrack)
{
  // Create trajectory only for charged particles
  fTrackingTimer->Start();
  G4double bgnz = (aTrack->GetPosition().z())/cm;
  G4double bgnr = (aTrack->GetPosition().perp())/cm;
  fTimerType    = 0;
  if( bgnr > 205.0 || ( bgnr > 45.0 && ( bgnz > 260.0 || bgnz < -260.0) ) ) {
    fTimerCal->Start();
    fTimerType=1;
  } else if ( ( bgnr <= 205. && bgnr > 40.0 ) 
	    && ( bgnz >= -260.0 && bgnz <= 260.0 ) ) {
    fTimerTracker->Start();
    fTimerType=2;
  }

  fCurrentTrack   = aTrack;
  fCurrentTrackID = aTrack->GetTrackID();
  if (fCurrentTrackID > fTrackCounts) fTrackCounts = fCurrentTrackID;

  // Reset current track ID for PreHit making upon starting of a new track

  using namespace std;
  vector<J4VSubTrackingAction *>::iterator iter;
  for (iter = fRegs.begin(); iter != fRegs.end(); iter++) {
    (*iter)->PreTrackDoIt(aTrack);
  }
  
#ifdef G4_STORE_TRAJECTORY
// if (fpTrackingManager && fpTrackingManager->GetStoreTrajectory()) { 
     if (fStoredTrajectoryID == 1) {
        // charged particle only
        if (aTrack->GetDefinition()->GetPDGCharge()) {
            fpTrackingManager->SetStoreTrajectory(true); 
        } else {
            fpTrackingManager->SetStoreTrajectory(false); 
        }
     } else if (fStoredTrajectoryID == 0 ) {
       fpTrackingManager->SetStoreTrajectory(false);
     } else if (fStoredTrajectoryID == 2 ) {
       fpTrackingManager->SetStoreTrajectory(true);
     }
// }
#endif
}

//=====================================================================
//* PostUserTrackingAction --------------------------------------------
void J4TrackingAction::PostUserTrackingAction(const G4Track* /* aTrack */)
{
  fTrackingTimer->Stop();
  if (fTimerType == 1) {
    fTimerCal->Stop();
  } else if (fTimerType == 2) {
    fTimerTracker->Stop();
  }
}

//=====================================================================
//* Clear -------------------------------------------------------------
void J4TrackingAction::Clear()
{
  using namespace std;
  vector<J4VSubTrackingAction *>::iterator iter;
  for (iter = fRegs.begin(); iter != fRegs.end(); iter++) {
    (*iter)->Clear();
  }
}

//=====================================================================
//* Add ---------------------------------------------------------------
void J4TrackingAction::Add(J4VSubTrackingAction *stap)
{
  std::vector<J4VSubTrackingAction *>::iterator iter;
  for(iter=fRegs.begin();iter!=fRegs.end();iter++) {
    if (*iter == stap) { 
      fRegs.erase(iter);
      break;
    }
  } 
  fRegs.push_back(stap);    
}
